
/*******************************************************************************
 * Copyright (c) 2015 Olivier Langella <Olivier.Langella@moulon.inra.fr>.
 *
 * This file is part of XTPcpp.
 *
 *     XTPcpp is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     XTPcpp is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with XTPcpp.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributors:
 *     Olivier Langella <Olivier.Langella@moulon.inra.fr> - initial API and
 *implementation
 ******************************************************************************/
#include "project.h"
#include "../input/xpipsaxhandler.h"
#include "../input/xtpxpipsaxhandler.h"
#include "peptidematch.h"
#include "proteinmatch.h"
#include <QDebug>

Project::Project()
{
}

Project::~Project()
{
  auto it = _identification_goup_list.begin();
  while(it != _identification_goup_list.end())
    {
      delete(*it);
      it++;
    }
}

bool
Project::checkPsimodCompliance() const
{
  return _peptide_store.checkPsimodCompliance();
}
void
Project::readResultFile(QString filename)
{
  IdentificationDataSourceSp ident_source =
    _identification_data_source_store.getInstance(filename);

  ident_source.get()->parseTo(this);
  qDebug() << "Project::readResultFile end";
}

const ProjectMode
Project::getProjectMode() const
{
  return _project_mode;
}
void
Project::setProjectMode(ProjectMode mode)
{
  _project_mode = mode;
}
std::vector<IdentificationGroup *>
Project::getIdentificationGroupList()
{
  return _identification_goup_list;
}

const std::vector<IdentificationGroup *>
Project::getIdentificationGroupList() const
{
  return _identification_goup_list;
}
const GroupingType
Project::getGroupingType() const
{
  return _grouping_type;
}

FastaFileStore &
Project::getFastaFileStore()
{
  return _fasta_file_store;
}

const FastaFileStore &
Project::getFastaFileStore() const
{
  return _fasta_file_store;
}
MsRunStore &
Project::getMsRunStore()
{
  return _msrun_store;
}
const MsRunStore &
Project::getMsRunStore() const
{
  return _msrun_store;
}
PeptideStore &
Project::getPeptideStore()
{
  return _peptide_store;
}

ProteinStore &
Project::getProteinStore()
{
  return _protein_store;
}

const ProteinStore &
Project::getProteinStore() const
{
  return _protein_store;
}

IdentificationDataSourceStore &
Project::getIdentificationDataSourceStore()
{
  return _identification_data_source_store;
}
const IdentificationDataSourceStore &
Project::getIdentificationDataSourceStore() const
{
  return _identification_data_source_store;
}
const AutomaticFilterParameters &
Project::getAutomaticFilterParameters() const
{
  return _automatic_filter_parameters;
}
void
Project::updateAutomaticFilters(
  const AutomaticFilterParameters &automatic_filter_parameters)
{

  qDebug() << "Project::updateAutomaticFilters begin";
  _automatic_filter_parameters = automatic_filter_parameters;
  for(auto &p_id_group : _identification_goup_list)
    {
      p_id_group->updateAutomaticFilters(_automatic_filter_parameters);
    }

  qDebug() << "Project::updateAutomaticFilters end";
}
ProjectSp
Project::makeProjectSp() const
{
  return std::make_shared<Project>(*this);
}

IdentificationGroup *
Project::newIdentificationGroup()
{
  _p_current_identification_group = new IdentificationGroup(this);
  _identification_goup_list.push_back(_p_current_identification_group);
  return _p_current_identification_group;
}
void
Project::readXpipFile(WorkMonitorInterface *p_monitor, QFileInfo xpip_fileinfo)
{
  qDebug() << "Project::readXpipFile begin";

  XpipSaxHandler *parser_java_xpip = new XpipSaxHandler(p_monitor, this);

  QXmlSimpleReader simplereader;
  simplereader.setContentHandler(parser_java_xpip);
  simplereader.setErrorHandler(parser_java_xpip);

  qDebug() << "Project::readXpipFile Read XPIP XML result file '"
           << xpip_fileinfo.absoluteFilePath() << "'";

  QFile qfile(xpip_fileinfo.absoluteFilePath());
  QXmlInputSource xmlInputSource(&qfile);

  if(simplereader.parse(xmlInputSource))
    {

      qfile.close();
    }
  else
    {
      qDebug() << parser_java_xpip->errorString();
      // throw PappsoException(
      //    QObject::tr("error reading tandem XML result file :\n").append(
      //         parser->errorString()));

      qfile.close();

      if(parser_java_xpip->isJavaXpip())
        {
          // delete parser_java_xpip;
          throw pappso::PappsoException(
            QObject::tr("Error reading %1 XPIP file :\n %2")
              .arg(xpip_fileinfo.absoluteFilePath())
              .arg(parser_java_xpip->errorString()));
        }
      else
        {
          XtpXpipSaxHandler *parser_xtp_xpip =
            new XtpXpipSaxHandler(p_monitor, this);

          QXmlSimpleReader simplereaderb;
          simplereaderb.setContentHandler(parser_xtp_xpip);
          simplereaderb.setErrorHandler(parser_xtp_xpip);

          qDebug() << "Project::readXpipFile Read XPIP XML result file '"
                   << xpip_fileinfo.absoluteFilePath() << "'";

          QFile qfileb(xpip_fileinfo.absoluteFilePath());
          QXmlInputSource xmlInputSourceb(&qfileb);

          if(simplereaderb.parse(xmlInputSourceb))
            {
              delete parser_xtp_xpip;
              qfileb.close();
            }
          else
            {
              if(parser_xtp_xpip->isXtpXpip())
                {
                  throw pappso::PappsoException(
                    QObject::tr("Error reading %1 XPIP file :\n %2")
                      .arg(xpip_fileinfo.absoluteFilePath())
                      .arg(parser_xtp_xpip->errorString()));
                }
              else
                {
                  qDebug() << parser_xtp_xpip->errorString();
                  throw pappso::PappsoException(
                    QObject::tr(
                      "Error reading %1 XPIP file :\n it is not an XPIP file")
                      .arg(xpip_fileinfo.absoluteFilePath()));
                }
            }
        }
    }

  delete parser_java_xpip;
}


void
Project::startGrouping(WorkMonitorInterface *p_work_monitor)
{
  for(IdentificationGroup *p_id_group : _identification_goup_list)
    {
      p_id_group->startGrouping(_grouping_type, p_work_monitor);
    }
}
void
Project::setLabelingMethodSp(LabelingMethodSp labeling_method_sp)
{
  _labeling_method_sp = labeling_method_sp;
  _peptide_store.setLabelingMethodSp(labeling_method_sp);
}

LabelingMethodSp
Project::getLabelingMethodSp() const
{
  return _labeling_method_sp;
}

bool
Project::hasPtmExperiment() const
{
  if(getIdentificationGroupList().at(0)->getPtmGroupingExperiment() != nullptr)
    return true;
  return false;
}

void
Project::getSameXicPeptideEvidenceList(
  std::vector<const PeptideEvidence *> &peptide_evidence_list,
  const MsRun *p_msrun, const PeptideXtp *p_peptide, unsigned int charge) const
{
  for(const IdentificationGroup *p_ident_group : _identification_goup_list)
    {
      p_ident_group->getSameXicPeptideEvidenceList(peptide_evidence_list,
                                                   p_msrun, p_peptide, charge);
    }
}
